@@ -36,13 +36,6 @@ defined('MOODLE_INTERNAL') || die();
 define('JCLIC_DEFAULT_JARBASE', 'http://clic.xtec.cat/dist/jclic');
 define('JCLIC_DEFAULT_LAP', 5);
 
-if (!isset($CFG->jclic_jarbase)) {
-    set_config('jclic_jarbase', JCLIC_DEFAULT_JARBASE);
-}
-if (!isset($CFG->jclic_lap)) {
-    set_config("jclic_lap", "5");
-}
-
 // JClic file types
 define('JCLIC_FILE_TYPE_LOCAL', 'local');
 define('JCLIC_FILE_TYPE_EXTERNAL', 'external');
@@ -61,7 +54,7 @@ require_once($CFG->dirroot.'/calendar/lib.php');
 
 /**
  * Returns the information on whether the module supports a feature
- * 
+ *
  * @todo: review features before publishing the module
  *
  * @see plugin_supports() in lib/moodlelib.php
@@ -82,7 +75,7 @@ function jclic_supports($feature) {
         case FEATURE_BACKUP_MOODLE2:          return true;
 //        case FEATURE_SHOW_DESCRIPTION:        return true;
 //        case FEATURE_ADVANCED_GRADING:        return true;
-        default:                        
+        default:
           if (defined('FEATURE_SHOW_DESCRIPTION') && $feature==FEATURE_SHOW_DESCRIPTION) return true;
           else return null;
     }
@@ -95,60 +88,19 @@ function jclic_supports($feature) {
  * (defined by the form in mod_form.php) this function
  * will create a new instance and return the id number
  * of the new instance.
- * 
+ *
  * @todo: create event (when timedue added)
  *
  * @param object $jclic An object from the form in mod_form.php
  * @param mod_jclic_mod_form $mform
  * @return int The id of the newly inserted jclic record
  */
-function jclic_add_instance(stdClass $jclic, mod_jclic_mod_form $mform = null) {
-    global $DB;
-        
-    $cmid = $jclic->coursemodule;
-    $jclic->timecreated = time();
-    if ($jclic->skin=='') $jclic->skin = "default";
+function jclic_add_instance(stdClass $data, mod_jclic_mod_form $mform = null) {
+    global $CFG;
+    require_once($CFG->dirroot . '/mod/jclic/locallib.php');
 
-    if ($mform->get_data()->filetype === JCLIC_FILE_TYPE_LOCAL) {
-        $jclic->url = $mform->get_data()->jclicfile;
-    } else{
-        $jclic->url = $jclic->jclicurl;
-    }
-    
-    if ($jclic->grade >=0 ) {
-        $jclic->maxgrade = $jclic->grade;
-    }
-    
-
-    $jclic->id = $DB->insert_record('jclic', $jclic);
-    // we need to use context now, so we need to make sure all needed info is already in db
-    $DB->set_field('course_modules', 'instance', $jclic->id, array('id'=>$cmid));
-    
-    // Store the JClic and verify
-    if ($mform->get_data()->filetype === JCLIC_FILE_TYPE_LOCAL) {
-        $filename = jclic_set_mainfile($jclic);
-        $jclic->url = $filename;
-        $DB->update_record('jclic', $jclic);
-    }
-    
-    if ($jclic->timedue) {
-        $event = new stdClass();
-        $event->name        = $jclic->name;
-        $event->description = format_module_intro('jclic', $jclic, $jclic->coursemodule);
-        $event->courseid    = $jclic->course;
-        $event->groupid     = 0;
-        $event->userid      = 0;
-        $event->modulename  = 'jclic';
-        $event->instance    = $jclic->id;
-        $event->eventtype   = 'due';
-        $event->timestart   = $jclic->timedue;
-        $event->timeduration = 0;
-
-        calendar_event::create($event);
-    }
-    jclic_grade_item_update($jclic);
-    
-    return $jclic->id;    
+    $jclic = new jclic(context_module::instance($data->coursemodule), null, null);
+    return $jclic->add_instance($data, true);
 }
 
 /**
@@ -162,61 +114,12 @@ function jclic_add_instance(stdClass $jclic, mod_jclic_mod_form $mform = null) {
  * @param mod_jclic_mod_form $mform
  * @return boolean Success/Fail
  */
-function jclic_update_instance(stdClass $jclic, mod_jclic_mod_form $mform = null) {
-    global $DB;
-
-    $jclic->timemodified = time();
-    $jclic->id = $jclic->instance;
-    if ($mform->get_data()->filetype === JCLIC_FILE_TYPE_LOCAL) {
-        $jclic->url = $mform->get_data()->jclicfile;
-    } else{
-        $jclic->url = $jclic->jclicurl;
-    }
-    if ($jclic->grade >=0 ) {
-        $jclic->maxgrade = $jclic->grade;
-    }    
-    
-    $result = $DB->update_record('jclic', $jclic);
-    if ($result && $mform->get_data()->filetype === JCLIC_FILE_TYPE_LOCAL) {
-        $filename = jclic_set_mainfile($jclic);
-        $jclic->url = $filename;
-        $result = $DB->update_record('jclic', $jclic);
-    }
-    
-    if ($result && $jclic->timedue) {
-        $event = new stdClass();
-        if ($event->id = $DB->get_field('event', 'id', array('modulename'=>'jclic', 'instance'=>$jclic->id))) {
-            $event->name        = $jclic->name;
-            $event->description = format_module_intro('jclic', $jclic, $jclic->coursemodule);
-            $event->timestart   = $jclic->timedue;
-
-            $calendarevent = calendar_event::load($event->id);
-            $calendarevent->update($event);
-        } else {
-            $event = new stdClass();
-            $event->name        = $jclic->name;
-            $event->description = format_module_intro('jclic', $jclic, $jclic->coursemodule);
-            $event->courseid    = $jclic->course;
-            $event->groupid     = 0;
-            $event->userid      = 0;
-            $event->modulename  = 'jclic';
-            $event->instance    = $jclic->id;
-            $event->eventtype   = 'due';
-            $event->timestart   = $jclic->timedue;
-            $event->timeduration = 0;
-
-            calendar_event::create($event);
-        }
-    } else {
-        $DB->delete_records('event', array('modulename'=>'jclic', 'instance'=>$jclic->id));
-    }  
-    
-    if ($result){
-        // get existing grade item
-        $result = jclic_grade_item_update($jclic);
-    }
-    
-    return $result;
+function jclic_update_instance(stdClass $data, $form) {
+    global $CFG;
+    require_once($CFG->dirroot . '/mod/jclic/locallib.php');
+    $context = context_module::instance($data->coursemodule);
+    $jclic = new jclic($context, null, null);
+    return $jclic->update_instance($data);
 }
 
 /**
@@ -225,47 +128,18 @@ function jclic_update_instance(stdClass $jclic, mod_jclic_mod_form $mform = null
  * Given an ID of an instance of this module,
  * this function will permanently delete the instance
  * and any data that depends on it.
- * 
- * @todo: delete event records (after adding this feature to the module)
  *
  * @param int $id Id of the module instance
  * @return boolean Success/Failure
  */
 function jclic_delete_instance($id) {
-    global $DB;
-    
-    if (!$jclic = $DB->get_record('jclic', array('id'=>$id))) {
-        return false;
-    }
-    
-    $result = true;
-    // Delete any dependent records
-    $rs =  $DB->get_records('jclic_sessions', array('jclicid' => $id));
-    foreach($rs as $session){
-        $DB->delete_records('jclic_activities', array('session_id' => $session->session_id));
-    }
+    global $CFG;
+    require_once($CFG->dirroot . '/mod/jclic/locallib.php');
+    $cm = get_coursemodule_from_instance('jclic', $id, 0, false, MUST_EXIST);
+    $context = context_module::instance($cm->id);
 
-    $DB->delete_records('jclic_sessions', array('jclicid' => $id));
-
-    // delete items from the gradebook
-    if(!jclic_grade_item_delete($jclic)){
-        $result = false;
-    }
-
-    /** TODO: // delete files associated with this jclic
-        $fs = get_file_storage();
-        if (! $fs->delete_area_files($context->id) ) {
-            $result = false;
-        }
-    **/
-
-    // delete events related with this instance
-    $DB->delete_records('event', array('modulename'=>'jclic', 'instance'=>$id));
-
-    // delete the instance
-    $DB->delete_records('jclic', array('id' => $id));
-        
-    return $result;
+    $jclic = new jclic($context, $cm, null);
+    return $jclic->delete_instance();
 }
 
 /**
@@ -281,39 +155,43 @@ function jclic_delete_instance($id) {
  * @param object $jclic
  * @return stdClass|null
  */
-function jclic_user_outline($course, $user, $mod, $jclic) {    
+function jclic_user_outline($course, $user, $mod, $jclic) {
     global $CFG;
-    
-    require_once("$CFG->libdir/gradelib.php");
-    $result = null;
-    
-    $grades = grade_get_grades($course->id, 'mod', 'jclic', $jclic->id, $user->id);
-    if (!empty($grades->items[0]->grades)) {
-        $grade = reset($grades->items[0]->grades);
-        $result = new stdClass();
-        $result->info = get_string('grade') . ': ' . $grade->str_long_grade;
 
-        //if grade was last modified by the user themselves use date graded. Otherwise use date submitted
-        if ($grade->usermodified == $user->id || empty($grade->datesubmitted)) {
-            $result->time = $grade->dategraded;
-        } else {
-            $result->time = $grade->datesubmitted;
-        }
+    require_once("$CFG->libdir/gradelib.php");
+
+    $gradinginfo = grade_get_grades($course->id, 'mod', 'jclic', $jclic->id, $user->id);
+    $gradingitem = $gradinginfo->items[0];
+    $gradebookgrade = $gradingitem->grades[$user->id];
+
+    if (empty($gradebookgrade->str_long_grade)) {
+        return null;
     }
-    return $result;    
+
+    $result = new stdClass();
+    $result->info = get_string('grade') . ': ' . $gradebookgrade->str_long_grade;
+
+    // If grade was last modified by the user themselves use date graded. Otherwise use date submitted
+    if ($gradebookgrade->usermodified == $user->id || empty($gradebookgrade->datesubmitted)) {
+        $result->time = $gradebookgrade->dategraded;
+    } else {
+        $result->time = $gradebookgrade->datesubmitted;
+    }
+    return $result;
 }
 
+
 /**
  * Prints a detailed representation of what a user has done with
  * a given particular instance of this module, for user activity reports.
- * 
+ *
  * @todo: implement
  *
  * @return string HTML
  */
-function jclic_user_complete($course, $user, $mod, $jclic) {
-    $outline = jclic_user_outline($course, $user, $mod, $jclic);
-    
+function jclic_user_complete($course, $user, $coursemodule, $jclic) {
+    $outline = jclic_user_outline($course, $user, $coursemodule, $jclic);
+
     print_r($outline->info);
     return true;
 }
@@ -323,7 +201,7 @@ function jclic_user_complete($course, $user, $mod, $jclic) {
  * Given a course and a time, this module should find recent activity
  * that has occurred in jclic activities and print it out.
  * Return true if there was output, or false is there was none.
- * 
+ *
  * @todo: implement
  *
  * @return boolean
@@ -334,7 +212,7 @@ function jclic_print_recent_activity($course, $viewfullnames, $timestart) {
 
 /**
  * Returns all activity in jclics since a given time
- * 
+ *
  * @todo: implement
  *
  * @param array $activities sequentially indexed array of objects
@@ -351,26 +229,14 @@ function jclic_get_recent_mod_activity(&$activities, &$index, $timestart, $cours
 
 /**
  * Prints single activity item prepared by {@see jclic_get_recent_mod_activity()}
- * 
+ *
  * @todo: implement
- * 
+ *
  * @return void
  */
 function jclic_print_recent_mod_activity($activity, $courseid, $detail, $modnames, $viewfullnames) {
 }
 
-/**
- * Function to be run periodically according to the moodle cron
- * This function searches for things that need to be done, such
- * as sending out mail, toggling flags etc ...
- *
- * @return boolean
- * @todo Finish documenting this function
- **/
-function jclic_cron () {
-    return true;
-}
-
 /**
  * Returns an array of users who are participanting in this jclic
  *
@@ -379,7 +245,7 @@ function jclic_cron () {
  * independient of his role (student, teacher, admin...). The returned
  * objects must contain at least id property.
  * See other modules as example.
- * 
+ *
  * @param int $jclicid ID of an instance of this module
  * @return boolean|array false if no participants, array of objects otherwise
  */
@@ -436,7 +302,7 @@ function jclic_get_extra_capabilities() {
  */
 function jclic_scale_used($jclicid, $scaleid) {
     global $DB;
-    
+
     $return = false;
     $rec = $DB->get_record('jclic', array('id'=>$jclicid,'grade'=>-$scaleid));
     if (!empty($rec) && !empty($scaleid)) {
@@ -475,7 +341,7 @@ function jclic_scale_used_anywhere($scaleid) {
  * @param mixed $grades optional array/object of grade(s); 'reset' means reset grades in gradebook
  * @return int 0 if ok
  */
-function jclic_grade_item_update(stdClass $jclic, $grades=NULL) {
+function jclic_grade_item_update(stdClass $jclic, $grades = null) {
     global $CFG;
     require_once($CFG->libdir.'/gradelib.php');
 
@@ -496,39 +362,26 @@ function jclic_grade_item_update(stdClass $jclic, $grades=NULL) {
         $params['scaleid']   = -$jclic->grade;
 
     } else {
-        $params['gradetype'] = GRADE_TYPE_NONE; // allow text comments only
+        $params['gradetype'] = GRADE_TYPE_TEXT; // allow text comments only
     }
 
-    if ($grades  === 'reset') {
+    if ($grades === 'reset') {
         $params['reset'] = true;
-        $grades = NULL;
+        $grades = null;
     }
-    
+
     grade_update('mod/jclic', $jclic->courseid, 'mod', 'jclic', $jclic->id, 0, $grades, $params);
 
     return true;
 }
 
-/**
- * Delete grade item for given jclic
- *
- * @global object
- * @param object $jclic object
- * @return object grade_item
- */
-function jclic_grade_item_delete($jclic) {
-    global $CFG;    
-    require_once($CFG->libdir.'/gradelib.php');
-
-    return grade_update('mod/jclic', $jclic->course, 'mod', 'jclic', $jclic->id, 0, NULL, array('deleted'=>1)) == GRADE_UPDATE_OK;
-}
 
 /**
  * Return grade for given user or all users.
  *
  * @todo: implement userid=0 (all users)
  * @todo: optimize this function (to avoid call jclic_get_sessions_summary or update only mandatory info)
- * 
+ *
  * @param object $jclic object
  * @param int $userid optional user id, 0 means all users
  * @return array array of grades, false if none
@@ -541,7 +394,7 @@ function jclic_get_user_grades($jclic, $userid=0) {
     if (! isset($jclic->id)) {
         return;
     }
-    
+
     $grades[$userid] = new stdClass();
     $sessions_summary = jclic_get_sessions_summary($jclic->id, $userid);
     $grades[$userid]->userid = $userid;
@@ -551,7 +404,7 @@ function jclic_get_user_grades($jclic, $userid=0) {
     $grades[$userid]->done = $sessions_summary->done;
     $grades[$userid]->rawgrade = 0;
     if ($jclic->avaluation=='score'){
-        $grades[$userid]->rawgrade = $sessions_summary->score;				
+        $grades[$userid]->rawgrade = $sessions_summary->score;
     }else{
         $grades[$userid]->rawgrade = $sessions_summary->solved;
     }
@@ -562,7 +415,7 @@ function jclic_get_user_grades($jclic, $userid=0) {
  * Update jclic grades in the gradebook
  *
  * Needed by grade_update_mod_grades() in lib/gradelib.php
- * 
+ *
  * @todo: Fix some problems (this function is not working when is called from beans.php)
  *
  * @param stdClass $jclic instance object with extra cmidnumber and modname property
@@ -590,7 +443,7 @@ function jclic_update_grades(stdClass $jclic, $userid = 0, $nullifnone=true) {
         $grade->userid   = $userid;
         $grade->rawgrade = NULL;
         jclic_grade_item_update($jclic, $grade);
-        
+
     } else {
         jclic_grade_item_update($jclic);
     }
@@ -680,7 +533,7 @@ function jclic_pluginfile($course, $cm, $context, $filearea, array $args, $force
     }
 
     require_login($course, true, $cm);
-    
+
     if (!has_capability('mod/jclic:view', $context)) {
         return false;
     }
@@ -699,38 +552,25 @@ function jclic_pluginfile($course, $cm, $context, $filearea, array $args, $force
         if ($file = $fs->get_file_by_hash(sha1($fullpath))) {
             break;
         }
-/*            
+/*
             $jclic = $DB->get_record('jclic', array('id'=>$cm->instance), 'id, legacyfiles', MUST_EXIST);
             if (!$file = jcliclib_try_file_migration('/'.$relativepath, $cm->id, $cm->course, 'mod_jclic', 'content', 0)) {
                 return false;
             }
             // file migrate - update flag
             $resource->legacyfileslast = time();
-            $DB->update_record('resource', $resource); 
+            $DB->update_record('resource', $resource);
  */
     } while (false);
 
     // finally send the file
-    send_stored_file($file, 86400, 0, $forcedownload);    
+    send_stored_file($file, 86400, 0, $forcedownload);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 // Navigation API                                                             //
 ////////////////////////////////////////////////////////////////////////////////
 
-/**
- * Extends the global navigation tree by adding jclic nodes if there is a relevant content
- *
- * This can be called by an AJAX request so do not rely on $PAGE as it might not be set up properly.
- *
- * @param navigation_node $navref An object representing the navigation tree node of the jclic module instance
- * @param stdClass $course
- * @param stdClass $module
- * @param cm_info $cm
- */
-function jclic_extend_navigation(navigation_node $navref, stdclass $course, stdclass $module, cm_info $cm) {
-}
-
 /**
  * Extends the settings navigation with the jclic settings
  *
@@ -741,6 +581,37 @@ function jclic_extend_navigation(navigation_node $navref, stdclass $course, stdc
  * @param navigation_node $jclicnode {@link navigation_node}
  */
 function jclic_extend_settings_navigation(settings_navigation $settingsnav, navigation_node $jclicnode=null) {
+    global $PAGE;
+
+    $keys = $jclicnode->get_children_key_list();
+    $beforekey = null;
+    $i = array_search('modedit', $keys);
+    if ($i === false and array_key_exists(0, $keys)) {
+        $beforekey = $keys[0];
+    } else if (array_key_exists($i + 1, $keys)) {
+        $beforekey = $keys[$i + 1];
+    }
+    if (has_capability('moodle/grade:viewall', $PAGE->context)){
+        $node = navigation_node::create(get_string('preview_jclic', 'jclic'),
+                new moodle_url('/mod/jclic/view.php', array('id'=>$PAGE->cm->id, 'action' => 'preview')),
+                navigation_node::TYPE_SETTING, null, 'mod_preview_jclic_preview',
+                new pix_icon('i/preview', ''));
+        $jclicnode->add_node($node, $beforekey);
+
+        $url = new moodle_url('/mod/jclic/report.php',
+                array('id' => $PAGE->cm->id, 'mode'=> 'normal'));
+        $reportnode = $jclicnode->add_node(navigation_node::create(get_string('results', 'jclic'), $url,
+                navigation_node::TYPE_SETTING,
+                null, null, new pix_icon('i/report', '')), $beforekey);
+        $reportnode->add_node(navigation_node::create(get_string('report_normal', 'jclic'), $url,
+                navigation_node::TYPE_SETTING,
+                null, null, new pix_icon('i/report', '')));
+        $url = new moodle_url('/mod/jclic/report.php',
+                array('id' => $PAGE->cm->id, 'mode'=> 'details'));
+        $reportnode->add_node(navigation_node::create(get_string('report_details', 'jclic'), $url,
+                navigation_node::TYPE_SETTING,
+                null, null, new pix_icon('i/report', '')));
+    }
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -779,9 +650,9 @@ function jclic_reset_gradebook($courseid) {
 function jclic_reset_userdata($data) {
     global $CFG, $DB;
 
-    $componentstr = get_string('modulenameplural', 'choice');
+    $componentstr = get_string('modulenameplural', 'jclic');
     $status = array();
- 
+
     if (!empty($data->reset_jclic_deleteallsessions)) {
         $params = array('courseid' => $data->courseid);
         $select = 'session_id IN'
@@ -794,15 +665,15 @@ function jclic_reset_userdata($data) {
             . " (SELECT j.id FROM {jclic} j"
             . " WHERE j.course = :courseid)";
         $DB->delete_records_select('jclic_sessions', $select, $params);
-        
+
         // remove all grades from gradebook
         if (empty($data->reset_gradebook_grades)) {
             jclic_reset_gradebook($data->courseid);
         }
-        
+
         $status[] = array('component'=>$componentstr, 'item'=>get_string('deleteallsessions', 'jclic'), 'error'=>false);
     }
- 
+
    return $status;
 }
 
@@ -814,7 +685,7 @@ function jclic_reset_userdata($data) {
 function jclic_reset_course_form_definition(&$mform) {
     $mform->addElement('header', 'jclicheader', get_string('modulenameplural', 'jclic'));
     $mform->addElement('checkbox', 'reset_jclic_deleteallsessions', get_string('deleteallsessions', 'jclic'));
-    
+
 }
 
 /**
@@ -823,3 +694,21 @@ function jclic_reset_course_form_definition(&$mform) {
 function jclic_reset_course_form_defaults($course) {
     return array('reset_jclic_deleteallsessions' => 1);
 }
+
+/**
+ * List the actions that correspond to a view of this module.
+ * This is used by the participation report.
+ * @return array
+ */
+function jclic_get_view_actions() {
+    return array('view');
+}
+
+/**
+ * List the actions that correspond to a post of this module.
+ * This is used by the participation report.
+ * @return array
+ */
+function jclic_get_post_actions() {
+    return array();
+}
\ No newline at end of file
